/*
 * Copyright (c) 2019 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * Notwithstanding the foregoing, you may not use, copy, modify, merge, publish,
 * distribute, sublicense, create a derivative work, and/or sell copies of the
 * Software in any work that is designed, intended, or marketed for pedagogical or
 * instructional purposes related to programming, coding, application development,
 * or information technology.  Permission for such use, copying, modification,
 * merger, publication, distribution, sublicensing, creation of derivative works,
 * or sale is expressly withheld.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package com.raywenderlich.android.ingredisearch.searchResults

import android.content.Intent

import android.view.View
import androidx.test.espresso.intent.rule.IntentsTestRule
import androidx.test.filters.LargeTest
import androidx.test.platform.app.InstrumentationRegistry

import androidx.test.rule.ActivityTestRule
import com.agoda.kakao.image.KImageView
import com.agoda.kakao.intent.KIntent
import com.agoda.kakao.recycler.KRecyclerItem
import com.agoda.kakao.recycler.KRecyclerView
import com.agoda.kakao.screen.Screen
import com.agoda.kakao.text.KTextView
import com.raywenderlich.android.ingredisearch.R
import com.raywenderlich.android.ingredisearch.recipe.RecipeActivity
import org.hamcrest.Matcher
import org.junit.Rule
import org.junit.Test


class Item(parent: Matcher<View>) : KRecyclerItem<Item>(parent) {
  val title = KTextView(parent) { withId(R.id.title) }
  val favButton = KImageView(parent) { withId(R.id.favButton) }
}

class SearchResultsScreen : Screen<SearchResultsScreen>() {
  val recycler: KRecyclerView = KRecyclerView({
    withId(R.id.list)
  }, itemTypeBuilder = {
    itemType(::Item)
  })
}

@LargeTest
class SearchResultsUITests {

  @Rule
  @JvmField
  var rule: IntentsTestRule<SearchResultsActivity> =
      object : IntentsTestRule<SearchResultsActivity>(SearchResultsActivity::class.java) {
        override fun getActivityIntent(): Intent {
          val targetContext = InstrumentationRegistry.getInstrumentation().targetContext
          val result = Intent(targetContext, SearchResultsActivity::class.java)
          result.putExtra("EXTRA_QUERY", "eggs, tomato")
          return result
        }
      }

  private val screen = SearchResultsScreen()

  @Test
  fun shouldRenderRecipesFromRepository() {
    screen {
      recycler {
        hasSize(10)
      }
    }
  }

  @Test
  fun shouldRenderTitleAndFavorite() {
    screen {
      recycler {
        for (i in 0..9) {
          scrollTo(i)
          childAt<Item>(i) {
            title.hasText("Title " + (i + 1))
            if (i != 1) {
              favButton.hasDrawable(R.drawable.ic_favorite_border_24dp)
            } else {
              favButton.hasDrawable(R.drawable.ic_favorite_24dp)
            }
          }
        }
      }
    }
  }

  @Test
  fun shouldChangeNotFavoriteToFavorite() {
    screen {
      recycler {
        scrollTo(0)
        childAt<Item>(0) {
          favButton.click()
          favButton.hasDrawable(R.drawable.ic_favorite_24dp)
        }
      }
    }
  }

  @Test
  fun shouldChangeFavoriteToNotFavorite() {
    screen {
      recycler {
        scrollTo(1)
        childAt<Item>(1) {
          favButton.click()
          favButton.hasDrawable(R.drawable.ic_favorite_border_24dp)
        }
      }
    }
  }

  @Test
  fun shouldLaunchRecipeActivity() {
    screen {
      recycler {
        scrollTo(0)
        childAt<Item>(0) {
          click()

          val recipeIntent = KIntent {
            hasComponent(RecipeActivity::class.java.name)
            hasExtra("EXTRA_URL", "https://www.simplyrecipes.com/recipes/homemade_pizza")
          }
          recipeIntent.intended()
        }
      }
    }
  }
}