/*
 * Copyright (c) 2019 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * Notwithstanding the foregoing, you may not use, copy, modify, merge, publish,
 * distribute, sublicense, create a derivative work, and/or sell copies of the
 * Software in any work that is designed, intended, or marketed for pedagogical or
 * instructional purposes related to programming, coding, application development,
 * or information technology.  Permission for such use, copying, modification,
 * merger, publication, distribution, sublicensing, creation of derivative works,
 * or sale is expressly withheld.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package com.raywenderlich.android.ingredisearch

import com.raywenderlich.android.ingredisearch.common.Recipe
import com.raywenderlich.android.ingredisearch.repository.RecipeRepository
import com.raywenderlich.android.ingredisearch.repository.RepositoryCallback

class IngredisearchTestApp : IngredisearchApp() {

  override fun getRecipeRepository(): RecipeRepository {
    return object : RecipeRepository {
      override fun addFavorite(item: Recipe) {}
      override fun removeFavorite(item: Recipe) {}
      override fun getFavoriteRecipes() = emptyList<Recipe>()
      override fun saveRecentIngredients(query: String) {}

      override fun getRecipes(query: String, callback: RepositoryCallback<List<Recipe>>) {
        val list = listOf(
            buildRecipe(1, false),
            buildRecipe(2, true),
            buildRecipe(3, false),
            buildRecipe(4, false),
            buildRecipe(5, false),
            buildRecipe(6, false),
            buildRecipe(7, false),
            buildRecipe(8, false),
            buildRecipe(9, false),
            buildRecipe(10, false)
        )
        callback.onSuccess(list)
      }

      override fun getRecentIngredients() =
          listOf("eggs", "ham", "onion", "tomato")
    }
  }

  private fun buildRecipe(id: Int, isFavorited: Boolean) =
      Recipe(id.toString(), "Title " + id.toString(), "", "", isFavorited)

}