/*
 * Copyright (c) 2019 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * Notwithstanding the foregoing, you may not use, copy, modify, merge, publish,
 * distribute, sublicense, create a derivative work, and/or sell copies of the
 * Software in any work that is designed, intended, or marketed for pedagogical or
 * instructional purposes related to programming, coding, application development,
 * or information technology.  Permission for such use, copying, modification,
 * merger, publication, distribution, sublicensing, creation of derivative works,
 * or sale is expressly withheld.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package com.raywenderlich.android.ingredisearch.search

import android.app.Activity.RESULT_OK
import android.content.Intent
import androidx.test.espresso.intent.rule.IntentsTestRule
import androidx.test.filters.LargeTest
import com.agoda.kakao.common.views.KView
import com.agoda.kakao.edit.KEditText
import com.agoda.kakao.intent.KIntent
import com.agoda.kakao.screen.Screen
import com.agoda.kakao.text.KButton
import com.raywenderlich.android.ingredisearch.R
import com.raywenderlich.android.ingredisearch.recentIngredients.RecentIngredientsActivity
import com.raywenderlich.android.ingredisearch.searchResults.SearchResultsActivity
import org.junit.Rule
import org.junit.Test

class SearchScreen : Screen<SearchScreen>() {
  val ingredients = KEditText { withId(R.id.ingredients) }
  val searchButton = KButton { withId(R.id.searchButton) }
  val snackbar = KView { withId(com.google.android.material.R.id.snackbar_text) }
  val recentButton = KView { withId(R.id.recentButton) }
}

@LargeTest
class SearchUITests {

  @Rule
  @JvmField
  var rule = IntentsTestRule(SearchActivity::class.java)

  private val screen = SearchScreen()

  @Test
  fun search_withEmptyText_shouldShowSnackbarError() {
    screen {
      searchButton.click()
      snackbar.isDisplayed()
    }
  }

  @Test
  fun search_withText_shouldNotShowSnackbarError() {
    screen {
      ingredients.typeText("eggs, ham, cheese")
      searchButton.click()
      snackbar.doesNotExist()
    }
  }

  @Test
  fun search_withText_shouldLaunchSearchResults() {
    screen {
      val query = "eggs, ham, cheese"
      ingredients.typeText(query)
      searchButton.click()

      val searchResultsIntent = KIntent {
        hasComponent(SearchResultsActivity::class.java.name)
        hasExtra("EXTRA_QUERY", query)
      }
      searchResultsIntent.intended()
    }
  }

  @Test
  fun choosingRecentIngredients_shouldSetCommaSeparatedIngredients() {
    screen {
      val recentIngredientsIntent = KIntent {
        hasComponent(RecentIngredientsActivity::class.java.name)
        withResult {
          withCode(RESULT_OK)
          withData(
              Intent().putStringArrayListExtra(
                  RecentIngredientsActivity.EXTRA_INGREDIENTS_SELECTED,
                  ArrayList(listOf("eggs", "onion"))
              )
          )
        }
      }
      recentIngredientsIntent.intending()

      recentButton.click()

      ingredients.hasText("eggs,onion")
    }
  }
}